
from ghidra.program.model.address import Address
from ghidra.program.model.mem import MemoryAccessException
from ghidra.util.task import ConsoleTaskMonitor
from ghidra.program.flatapi import FlatProgramAPI


settings = {
  ">DEFAULT_ACCELERATION":  0x08009408,
  ">DEFAULT_E_MAX_FEEDRATE":  0x080093e4,
  ">DEFAULT_E_STEPS_PER_UNIT":  0x080093d4,
  ">DEFAULT_EJERK":  0x08009404,
  ">DEFAULT_Kd":  0x08009420,
  ">DEFAULT_Ki":  0x08009418,
  ">DEFAULT_Kp":  0x08009414,
  ">DEFAULT_RETRACT_ACCELERATION":  0x0800940c,
  ">DEFAULT_TRAVEL_ACCELERATION":  0x08009410,
  ">DEFAULT_X_MAX_FEEDRATE":  0x080093d8,
  ">DEFAULT_X_STEPS_PER_UNIT":  0x080093c8,
  ">DEFAULT_XJERK":  0x080093f8,
  ">DEFAULT_Y_MAX_FEEDRATE":  0x080093dc,
  ">DEFAULT_Y_STEPS_PER_UNIT":  0x080093cc,
  ">DEFAULT_YJERK":  0x080093fc,
  ">DEFAULT_Z_MAX_FEEDRATE":  0x080093e0,
  ">DEFAULT_Z_STEPS_PER_UNIT":  0x080093d0,
  ">DEFAULT_ZJERK":  0x08009400,
  ">EXTRUDE_MINTEMP":  0x08009398,
  ">FILAMENT_CHANGE_X_POS":  0x08009370,
  ">FILAMENT_CHANGE_Y_POS":  0x08009374,
  ">FILAMENT_CHANGE_Z_ADD":  0x08009378,
  ">HOMING_FEEDRATE_XY":  0x080093c0,
  ">HOMING_FEEDRATE_Z":  0x080093c4,
  ">X_MAX_POS":  0x08009364,
  ">X_MIN_POS":  0x08009358,
  ">Y_MAX_POS":  0x08009368,
  ">Y_MIN_POS":  0x0800935c,
  ">Z_MAX_POS":  0x0800936c,
  ">Z_MIN_POS":  0x08009360,
  ">INVERT_E0_DIR":  0x08009394,
  ">INVERT_X_DIR":  0x08009388,
  ">INVERT_Y_DIR":  0x0800938c,
  ">INVERT_Z_DIR":  0x08009390,
  ">PIDTEMP":  0x080093b0,
  ">THERMAL_PROTECTION_HYSTERESIS":  0x080093a4,
  ">WATCH_TEMP_INCREASE":  0x080093ac,
  ">CURRENT_VREF_E":  0x08009384,
  ">CURRENT_VREF_XY":  0x0800937c,
  ">CURRENT_VREF_Z":  0x08009380,
  ">HEATER_0_MAXTEMP":  0x0800939c,
  ">THERMAL_PROTECTION_PERIOD":  0x080093a0,
  ">WATCH_TEMP_PERIOD":  0x080093a8,
  ">X_HOME_DIR":  0x080093b4,
  ">Y_HOME_DIR":  0x080093b8,
  ">Z_HOME_DIR":  0x080093bc,
  ">DEFAULT_E_MAX_ACCELERATION":  0x080093f4,
  ">DEFAULT_X_MAX_ACCELERATION":  0x080093e8,
  ">DEFAULT_Y_MAX_ACCELERATION":  0x080093ec,
  ">DEFAULT_Z_MAX_ACCELERATION":  0x080093f0,
}


def bytes_to_int_le(byte_array):
    result = 0
    for i in range(len(byte_array)):
        result |= (byte_array[i] & 0xFF) << (8 * i)
    return result

for name, addr_val in settings.items():
    addr = toAddr(addr_val)
    try:
        # Read 4 bytes at addr (pointer)
        ptr_bytes = getBytes(addr, 4)
        ptr_value = bytes_to_int_le(ptr_bytes)

        # Dereference pointer and read 4 bytes
        deref_addr = toAddr(ptr_value)
        value_bytes = getBytes(deref_addr, 4)
        final_value = bytes_to_int_le(value_bytes)

        # Output the result
        print(name + " at " + hex(addr_val) + " -> " + hex(ptr_value) + " -> " + str(final_value))
    except MemoryAccessException as e:
        print("Memory access failed at " + hex(addr_val) + " for " + name + ": " + str(e))
